//
//  RootViewController.m
//  uPnp
//
//  Created by rk on 2/01/11.
//  Copyright 2011 Interchart. All rights reserved.
//


#import "uPnpAppDelegate.h"
#import "RootViewController.h"
#import "DetailViewController.h"
#import "MediaServerViewController.h"


@implementation RootViewController

@synthesize popoverController;
@synthesize _upnpManager;
@synthesize _detailViewController;
@synthesize _mediaServerViewController;
@synthesize _devices;

#pragma mark -
#pragma mark showMediaServerView

- (IBAction)showMediaServerView: (id)sender {
    if (_mediaServerViewController == nil) {
        _mediaServerViewController = [[MediaServerViewController alloc] init ];
    }
//    [self.view addSubview: _mediaServerViewController.view];
//    [self.view addSubview:msView];
    if(![popoverController isPopoverVisible]){
		popoverController = [[[UIPopoverController alloc] initWithContentViewController:_mediaServerViewController] retain];
		
		[popoverController setPopoverContentSize:_mediaServerViewController.view.bounds.size];
		[popoverController presentPopoverFromBarButtonItem:sender permittedArrowDirections:UIPopoverArrowDirectionAny animated:YES];
		
		// Or use the following line to display it from a given rectangle
		//[popoverController presentPopoverFromRect:CGRectMake(10, 10, 100, 100) inView:self.view permittedArrowDirections:UIPopoverArrowDirectionAny animated:YES];
//		[popoverController presentPopoverFromRect:[sender frame] inView:self.view permittedArrowDirections:UIPopoverArrowDirectionUp animated:YES];
	} else {
		[popoverController dismissPopoverAnimated:YES];
	}

    
    
    
}

#pragma mark -
#pragma mark Discover Devices

- (IBAction) discoverDevices : (id)sender{
	
	if (_upnpManager == nil) {
		_upnpManager = [[[UpnpManager alloc] init] retain];
	}
	
	if (_hud != nil) return; // only allow one discoverDevices request at a time.
	
	_hud = [[MBProgressHUD alloc] initWithView:_detailViewController.view];
	// Add HUD to screen
    [_detailViewController.view addSubview:_hud];
	
    // Register for HUD callbacks so we can remove it from the window at the right time
    _hud.delegate = self;
	
	
	// Clear down the list of devices 
	if (_devices != nil) {
		[_devices release];
	}
	_devices = [[NSMutableArray array] retain];
	//[_devices retain];
	_discoveringDevices = true;
	[self.tableView reloadData];
	
	// Show the HUD while the provided method executes in a new thread
    [_hud showWhileExecuting:@selector(doDiscoverDevices) onTarget:self withObject:nil animated:YES];
	
}

// This method gets executed in a new thread, not main thread
- (void) doDiscoverDevices {
	_discoveringDevices = true;
	@synchronized (self) {
		_temporaryDevices = [_upnpManager discoverDevices];
	}
	_discoveringDevices = false;
}
	

// This method gets executed in main thread
- (void)discoverDevicesCompletion {
    NSIndexPath *currentSelection = [self.tableView indexPathForSelectedRow];
    if (currentSelection != nil) {
        [self.tableView deselectRowAtIndexPath:currentSelection animated:NO];
    }    
	
	if (_devices != nil) {
		[_devices release];
	}
	
	@synchronized (self){
	_devices = _temporaryDevices;
	_temporaryDevices = nil;
	}
	
	UpnpDevice *device;
	for (device in _devices) {
		device.delegate = self;
		[device fetchDeviceInfo];
	}
	
	//[_upnpManager fetchDeviceInfos: _devices];

	[self.tableView reloadData];
}

#pragma mark -
#pragma mark unsynthesized Properties

- (HTTPServer *)httpServer {
    uPnpAppDelegate *appDelegate = (uPnpAppDelegate *)[[UIApplication sharedApplication] delegate];
    HTTPServer *httpServer = appDelegate.httpServer;
    return httpServer;
}


#pragma mark -
- (NSMutableArray *)rendererDevices {
	NSMutableArray *result = [[[NSMutableArray alloc] init] autorelease];
	for (UpnpDevice *device in _devices) {
		if (device._deviceType == Renderer) {
			[result addObject:device];
		}
	}
	return result;
}

- (NSMutableArray *)serverDevices {
	NSMutableArray *result = [[[NSMutableArray alloc] init] autorelease];
	for (UpnpDevice *device in _devices) {
		if (device._deviceType == Server) {
			[result addObject:device];
		}
	}
	return result;
}

- (NSMutableArray *)otherDevices {
	NSMutableArray *result = [[[NSMutableArray alloc] init] autorelease];
	for (UpnpDevice *device in _devices) {
		if ((device._deviceType != Server) &&
			(device._deviceType != Renderer)) {
			[result addObject:device];
		}	
	}
	return result;
}

#pragma mark -
#pragma mark UpnpDeviceDelegate methods
- (void) upnpDeviceDisplayNameDidChangeValue:(NSNotification *)notification {
	NSLog(@"%@: upnpDeviceDisplayNameDidChangeValue: %@", self, [notification object]);
	// KLUDGE ALERT: we reload all the table entries whenever any one of them changes:
	[self.tableView reloadData];
}


#pragma mark -
#pragma mark MBProgressHUDDelegate methods
- (void)hudWasHidden {
	// Remove HUD from screen when the HUD was hidden
    [_hud removeFromSuperview];
    [_hud release];
	_hud = nil;
	
	[self discoverDevicesCompletion];
}


#pragma mark -
#pragma mark View lifecycle

- (void)viewDidLoad {
    [super viewDidLoad];
    self.clearsSelectionOnViewWillAppear = NO;
    self.contentSizeForViewInPopover = CGSizeMake(320.0, 600.0);
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(httpServerAddressesNotification:) name:HTTP_SERVER_ADDRESS_RESOLVED_NOTIFICATION_NAME object:nil];

}

- (void)httpServerAddressesNotification:(NSNotification *) notification
{
	NSLog(@"httpServerAddressesNotification:");

    [self.tableView reloadData]; // Put details of local httpServer in tableView
}

/*
- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
}
*/
/*
- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
}
*/
/*
- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
}
*/
/*
- (void)viewDidDisappear:(BOOL)animated {
    [super viewDidDisappear:animated];
}
*/

// Ensure that the view controller supports rotation and that the split view can therefore show in both portrait and landscape.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    return YES;
}


#pragma mark -
#pragma mark Table view data source

#define RENDERER_SECTION_NR 0
#define SERVER_SECTION_NR 1
#define OTHER_DEVICES_SECTION_NR 2
#define HTTP_SERVER_SECTION_NR 3
#define NR_SECTIONS 4

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)sectionNr {
	switch(sectionNr) {
		case RENDERER_SECTION_NR:
			return @"Renderers";
			break;
		case SERVER_SECTION_NR:
			return @"Servers";
			break;
		case HTTP_SERVER_SECTION_NR:
			return @"Local HTTP Server";
			break;
	}
	return @"Other Devices"; // default (fallthru)
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)aTableView {
    // Return the number of sections.
    return NR_SECTIONS;
}


- (NSInteger)tableView:(UITableView *)aTableView numberOfRowsInSection:(NSInteger)sectionNr {
    // Return the number of rows in the section.
	int result = 0;
	switch (sectionNr) {
		case RENDERER_SECTION_NR:
			result = [self.rendererDevices count];
			break;
		case SERVER_SECTION_NR:
			result = [self.serverDevices count];
			break;
		case HTTP_SERVER_SECTION_NR:
            result = self.httpServer ? 1 : 0;
			break;
		case OTHER_DEVICES_SECTION_NR:
			result = [self.otherDevices count];
			break;
	}
	return result;
}


- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    static NSString *CellIdentifier = @"CellIdentifier";
    
    // Dequeue or create a cell of the appropriate type.
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier] autorelease];
        cell.accessoryType = UITableViewCellAccessoryNone;
    }
    
    // Configure the cell.
    cell.textLabel.text = @"";
    //cell.imageView.image = [UIImage imageNamed:@"tv_72x72.png"];
    cell.imageView.image = nil;

	int sectionNr = indexPath.section;
	int rowNr = indexPath.row;
	if (_discoveringDevices) {
		cell.textLabel.text = @"Discovering devices...";
	} else {
		switch (sectionNr) {
			case RENDERER_SECTION_NR:
				if (rowNr < [self.rendererDevices count]) {
					cell.textLabel.text = [[self.rendererDevices objectAtIndex:rowNr] displayName];
                    NSData *iconImageData = [[self.rendererDevices objectAtIndex:rowNr] iconImageData];
                    if (iconImageData) {
                        cell.imageView.image = [UIImage imageWithData:iconImageData];
                    }
				}
				break;
			case SERVER_SECTION_NR:
				if (rowNr < [self.serverDevices count]) {
					cell.textLabel.text = [[self.serverDevices objectAtIndex:rowNr] displayName];
                    NSData *iconImageData = [[self.serverDevices objectAtIndex:rowNr] iconImageData];
                    if (iconImageData) {
                        cell.imageView.image = [UIImage imageWithData:iconImageData];
                    }
				}
				break;
			case HTTP_SERVER_SECTION_NR:
				if (self.httpServer) {
					cell.textLabel.text = [NSString stringWithFormat:@"%@", self.httpServer.baseUrl];
				}
				break;
			case OTHER_DEVICES_SECTION_NR:
				if (rowNr < [self.otherDevices count]) {
					cell.textLabel.text = [[self.otherDevices objectAtIndex:rowNr] displayName];
				}
				break;
			default:
				break;
		}
	}
	
    return cell;
}


/*
// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
    // Return NO if you do not want the specified item to be editable.
    return YES;
}
*/


/*
// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
    
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        // Delete the row from the data source
        [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:YES];
    }   
    else if (editingStyle == UITableViewCellEditingStyleInsert) {
        // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view
    }   
}
*/


/*
// Override to support rearranging the table view.
- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath {
}
*/


/*
// Override to support conditional rearranging of the table view.
- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath {
    // Return NO if you do not want the item to be re-orderable.
    return YES;
}
*/


#pragma mark -
#pragma mark Table view delegate

- (void)tableView:(UITableView *)aTableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    
    /*
     When a row is selected, set the detail view controller's detail item to the item associated with the selected row.
     */
	int sectionNr = indexPath.section;
	int rowNr = indexPath.row;
	id detailItem = nil;
    switch (sectionNr) {
        case RENDERER_SECTION_NR:
            detailItem = [self.rendererDevices objectAtIndex:rowNr];
            break;
        case SERVER_SECTION_NR:
            detailItem = [self.serverDevices objectAtIndex:rowNr];
            break;
        case HTTP_SERVER_SECTION_NR:
            detailItem = self.httpServer;
            break;
        case OTHER_DEVICES_SECTION_NR:
            detailItem = [self.otherDevices objectAtIndex:rowNr];
            break;
            
    }
    _detailViewController.detailItem = detailItem;
}


#pragma mark -
#pragma mark Memory management

-(id) init {
    self = [super init];
    _mediaServerViewController = [[[MediaServerViewController alloc] init] retain];
    return self;
}

- (void)didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Relinquish ownership any cached data, images, etc. that aren't in use.
}

- (void)viewDidUnload {
    // Relinquish ownership of anything that can be recreated in viewDidLoad or on demand.
    // For example: self.myOutlet = nil;
}


- (void)dealloc {
    [_mediaServerViewController release];
	[_upnpManager release];
    [_detailViewController release];
    [super dealloc];
}


@end

