//
//  UpnpDeviceInfo.m
//  uPnp2
//
//  Created by rk on 17/01/11.
//  Copyright 2011 Interchart. All rights reserved.
//

#import "UpnpDeviceInfo.h"
#import "UpnpDevice.h"


@implementation UpnpDeviceInfo

@synthesize device;
//@synthesize deviceTypeSchema, friendlyName, manufacturer, modelName, UDN;
@synthesize services;


+(NSString *)stringValueOfChildElement:(GDataXMLElement *)element: (NSString *)childElementName {
    NSString *result = @"";
//    NSLog(@"stringValueOfElement: node XMLString:%@", [element XMLString]);
    NSError *error = nil;
    NSArray *childElements=[element elementsForName:childElementName];
    if (error) {
        NSLog(@"erorr: %@", error);
    }
    if (childElements && ([childElements count] >0)) {
        result = [[childElements objectAtIndex:0] stringValue];
    }
    return result;
}

- (id) initWithXml:(id) ownerDeviceXmlData: (NSData *)xml {
	self = [super init];
	device = ownerDeviceXmlData;
	if (!xml) return self;
	
    NSLog(@"UpnpDeviceInfo:initWithXml, xml= %@", [Util dataToString:xml]);
	services = [[NSMutableArray alloc] init];
    icons = [[NSMutableArray alloc] init];
    
    NSError *error;
    parsedXml = [[GDataXMLDocument alloc] initWithData:xml options:0 error:&error];
    
    if (parsedXml) {
        NSLog(@"UpnpDeviceInfo: parsedXml:%@", parsedXml);
        NSArray* serviceNodes = [parsedXml nodesForXPath:@"//upnpDevice:root/upnpDevice:device/upnpDevice:serviceList/upnpDevice:service"
                                       namespaces:[UpnpAvService namespaceDict]
                                            error:nil];
        for (GDataXMLElement *element in serviceNodes) {
            UpnpAvService *service = [[UpnpAvService alloc] initWithDevice:device];
            service.serviceType = [UpnpDeviceInfo stringValueOfChildElement:element :@"serviceType"];
            service.serviceId = [UpnpDeviceInfo stringValueOfChildElement:element :@"serviceId"];
            service.scpdUrl = [UpnpDeviceInfo stringValueOfChildElement:element :@"SCPDURL"];
            service.controlURL = [UpnpDeviceInfo stringValueOfChildElement:element :@"controlURL"];
            service.eventSubURL = [UpnpDeviceInfo stringValueOfChildElement:element :@"eventSubURL"];
            [services addObject:service];
        }
        
        NSArray* iconNodes = [parsedXml nodesForXPath:@"//upnpDevice:root/upnpDevice:device/upnpDevice:iconList/upnpDevice:icon"
                                              namespaces:[UpnpAvService namespaceDict]
                                                   error:nil];
        
        for (GDataXMLElement *element in iconNodes) {
            DeviceIconModel *icon = [[DeviceIconModel alloc] initWithXmlElement:element];
            [icons addObject:icon];
        }
    }
    
//	NSXMLParser *parser = [[NSXMLParser alloc] initWithData:xml];
//	[parser setDelegate:self];
//	[parser parse];
//	[parser release];
	return self;
}


#pragma mark -
#pragma mark readonly properties
// utility method
-(NSString *)stringValueForXpath: (NSString *)xpath {
    NSString *result = @"";
    NSError *error;
    NSArray *nodes = [parsedXml nodesForXPath:xpath 
                                        namespaces:[UpnpAvService namespaceDict] 
                                             error:&error];
    if (nodes && ([nodes count] > 0)) {
        result = [[nodes objectAtIndex:0] stringValue];  
    }
    return result;

}


-(NSString *)deviceTypeSchema {
    return [self stringValueForXpath:@"//upnpDevice:root/upnpDevice:device/upnpDevice:deviceType"];
}

-(NSString *)friendlyName{
    return [self stringValueForXpath:@"//upnpDevice:root/upnpDevice:device/upnpDevice:friendlyName"];
}

-(NSString *)manufacturer {
    return [self stringValueForXpath:@"//upnpDevice:root/upnpDevice:device/upnpDevice:manufacturer"];
}

-(NSString *)modelName{
    return [self stringValueForXpath:@"//upnpDevice:root/upnpDevice:device/upnpDevice:modelName"];
}

-(NSString *)UDN {
    return [self stringValueForXpath:@"//upnpDevice:root/upnpDevice:device/upnpDevice:UDN"];
}


-(DeviceIconModel *)preferredIcon {
    DeviceIconModel *result = nil;
    if (icons && ([icons count] >0)) {
        // sortedIcons is sorted in descening height:
        NSArray *sortedIcons = [icons sortedArrayUsingSelector:@selector(compareByHeightReverse:)];
        for (DeviceIconModel *icon in sortedIcons) {
            if (icon.height < 100) {
                return icon;
//                break;
            }
        }
        // No icons smaller than 100, just return the smallest (last in sorted list):
        return [sortedIcons objectAtIndex:[sortedIcons count]-1];
    }
    return result;
}


//#pragma mark -
//#pragma mark NSXMLParserDelegate methods
//- (void) parser:(NSXMLParser *)parser didStartElement:(NSString *)elementName
//				namespaceURI:(NSString *)namespaceURI
//				qualifiedName:(NSString *)qualifiedName
//				attributes:(NSDictionary *)attributeDict {
//	// deviceTypeSchema, friendlyName, manufacturer, modelName, UDN;
//	if ([elementName isEqual:@"deviceType"]) {
//		deviceTypeSchema = [[NSMutableString alloc] init];
//		currentParserField = deviceTypeSchema;
//	} else
//	if ([elementName isEqual:@"friendlyName"]) {
//		friendlyName = [[NSMutableString alloc] init];
//		currentParserField = friendlyName;
//	} else
//	if ([elementName isEqual:@"manufacturer"]) {
//		manufacturer = [[NSMutableString alloc] init];
//		currentParserField = manufacturer;
//	} else 
//	if ([elementName isEqual:@"modelName"]) {
//		modelName = [[NSMutableString alloc] init];
//		currentParserField = modelName;
//	} else 
//	if ([elementName isEqual:@"UDN"]) {
//		UDN = [[NSMutableString alloc] init];
//		currentParserField = UDN;
//	} else
//	if ([elementName isEqual:@"service"]) {
//		serviceBeingParsed = [[UpnpAvService alloc] initWithDevice:device];
//		servicesBuffer = [[NSMutableString alloc] init];
//		currentParserField = servicesBuffer;
//	} else
//	if (([elementName isEqual:@"serviceType"]) ||
//		([elementName isEqual:@"serviceId"]) ||
//		([elementName isEqual:@"SCPDURL"]) ||
//		([elementName isEqual:@"controlURL"]) ||
//		([elementName isEqual:@"eventSubURL"]))	{
//		[servicesBuffer setString:@""];
//		currentParserField = servicesBuffer;
//	}
//	//NSLog(@"elementName:%@, qualifiedName:%@", elementName, qualifiedName);
//
//}
//
//- (void) parser:(NSXMLParser *)parser foundCharacters:(NSString *)string {
//	[currentParserField appendString:string];
//}
//
//- (void) parser:(NSXMLParser *)parser didEndElement:(NSString *)elementName
//				namespaceURI:(NSString *)namespaceURI
//				qualifiedName:(NSString *)qualfilifiedName {
//	if ([elementName isEqual:@"serviceType"]) {
//		serviceBeingParsed.serviceType = [servicesBuffer copy];
//		[servicesBuffer setString:@""];
//	}
//	if ([elementName isEqual:@"serviceId"]) {
//		serviceBeingParsed.serviceId = [servicesBuffer copy];
//		[servicesBuffer setString:@""];
//	}
//	if ([elementName isEqual:@"SCPDURL"]) {
//		serviceBeingParsed.scpdUrl = [servicesBuffer copy];
//		[servicesBuffer setString:@""];
//	}
//	if ([elementName isEqual:@"controlURL"]) {
//		serviceBeingParsed.controlURL = [servicesBuffer copy];
//		[servicesBuffer setString:@""];
//	}
//	if ([elementName isEqual:@"eventSubURL"]) {
//		serviceBeingParsed.eventSubURL = [servicesBuffer copy];
//		[servicesBuffer setString:@""];
//	}
//	if ([elementName isEqual:@"service"]) {
//		NSLog(@"%@: parser didEndElement '%@', servicesBuffer='%@'", 
//			  self, 
//			  elementName,
//			  servicesBuffer);
//		[servicesBuffer release];
//		[services addObject:serviceBeingParsed];
//		NSLog(@" services=%@, [services count] = %d", services, [services count]);
//	}
//	currentParserField = nil;
//}
	


#pragma mark -

- (void) dealloc {
//	[deviceTypeSchema release];
//	[friendlyName release];
//	[manufacturer release];
//	[modelName release];
//	[UDN release];
	for (id service in services) {
		[service release];
	}
	[services release];
    
	for (id icon in icons) {
		[icon release];
	}
	[icons release];
    
    [parsedXml release];
	
	[super dealloc];
}

- (NSString *)description {
	return [NSString stringWithFormat:@"%@, %@", [self friendlyName], [self UDN]];
}

@end
